# protoparser.js

Protoparser.js – легковесный веб-движок (фреймворк) для [интерактивных текстовых игр](https://ru.wikipedia.org/wiki/Interactive_fiction) с вводом команд («парсеров»).

Приставка *«прото»* в названии движка, в переводе с древнегреческого, означает «первый». Интерфейс командной строки протопарсера оперирует всего двумя понятиями: ГЛАГОЛ и СУЩЕСТВИТЕЛЬНОЕ. В этом он похож на первые текстовые игры-приключения. Основная цель проекта, в первую очередь, – не достичь возможностей каких-то существующих парсерных движов, а создать инструмент, который бы ни в чем не  ограничивал автора, был максимально простым и открытым, при этом, оставался бы незаметным. 

## Основные особенности и возможности protoparser.js

* Protoparser.js платформонезависимый. Вы можете запускать и писать игры на любом устройстве, в котором есть браузер с поддержкой JavaScript. Не нужно компилировать исходники, устанавливать интерпретаторы, библиотеки, и пр. Все, что нужно, чтобы начать писать игру – любая программа в которой можно набрать и сохранить текст. Протопарсер работает даже на старых устройствах.
+ При желании, вы можете писать игру прямо в браузере, переключившись в «режим разработчика», если ваш браузер поддерживает такой режим.
* Законченную игру можно выложить на сайт, где она будет доступна онлайн. Вы также можете запускать ее локально. 
+ В протопарсере есть система автотестирования, которая избавит вас от необходимости после каждой правки вручную вбивать команды и проверять как все работает. Кроме того, благодаря наличию во многих современных браузерах «режима разработчика» вы можете в реальном времени проверять и изменять состояние игровых объектов.
* Если вы даже не собираетесь писать игры на протопарсере, вы, тем не менее, можете использовать его как инструмент для быстрого прототипирования.
+ Protoparser.js написан на чистом JavaScript'е, и, кроме эмулятора терминала [terminal.js](../terminaljs/), не использует иных библиотек или зависимостей. Protoparser.js является одним из самых легковесных парсерных движков на сегодняшний день. Его размер составляет всего несколько десятков килобайт.
* В протопарсере доступны различные системы управления игрой помимо традиционного ввода команд: «горячие клавиши», делающие управление похожим на то, какое используется в традиционных компьютерных играх; история команд, дающая быстрый доступ к раннее введенным командам; а также сокращенный ввод, позволяющий использовать команды, состоящие всего из нескольких символов.
+ Поскольку protoparser.js является веб-приложением вы можете использовать все возможности HTML, CSS и JS в своих играх: вставлять картинки, музыку, звуки, видео; устанавливать стили оформления; подключать дополнительные js-библиотеки, и многое другое.
* Protoparser.js выполняется полностью на стороне клиента, и там же хранит все данные.
+ Protoparser.js поддерживает мульти-сохранение и загрузку игр.
* Игры для протопарсера пишутся на языке JavaScript. В настоящее время, это один из самых распространенных языков программирования. Даже если вы напишите всего одну игру, опыт приобретенный при ее написании, может помочь вам, если вы в дальнейшем захотите заняться программированием на этом языке.
+ Protoparser.js позволяет автору создавать новые команды и менять параметры стандартных команд.
* Protoparser.js распространяется на условиях лицензии [MIT](../LICENSE.txt). Вы можете изменять и использовать его в своих проектах.

## Системные требования

Большинство браузеров.

## Запуск

Запустите файл **index.html**.

## Демонстрационная игра

Protoparser.js распространяется вместе с игрой «[Фантазия](example)». Данная игра была написана для демонстрации возможностей движка и используется в качестве обучающего примера в [руководстве автора](manual/protoparser_manual.md). Игра также доступна **[онлайн](https://johnbrown90210.gitlab.io/Fantasia)**.

## Руководство

Для того, чтобы начать писать игры на протопарсере необходимо ознакомиться с [руководством автора](manual/protoparser_manual.md). Кроме того, желательно, обладать хотя бы начальными знаниями JavaScript'а.

## Актуальная версия

Вы можете [скачать актуальную версию protoparser.js](https://gitlab.com/johnbrown90210/protoparser.js/-/archive/master/protoparser.js-master.zip) и посмотреть [список изменений](CHANGELOG.txt) на странице проекта https://johnbrown90210.gitlab.com/protoparser.js.

## ЧАВО

* **При загрузке/сохранении интерпретатор пишет «Ошибка загрузки/сохранения.»**   
  Причины могут быть следующие: 
  + В вашем браузере отсутствует объект `localStorage`, либо браузер не предоставляет к нему доступ.
  + Вы очистили кэш браузера после того как сохранили игру.
  + Вы сохранили игру, находясь в режиме «инкогнито».
  + Вы сохранили игру под другим именем.
*  **Игра не понимает мои команды.**  
Команда должна состоять не более, чем из двух слов. Регистр не учитывается. Полный список распознаваемых команд можно посмотреть в «[Приложении 1. Стандартные функции-обработчики (API)](https://gitlab.com/johnbrown90210/protoparser.js/blob/master/manual/protoparser_manual.md#%D0%BF%D1%80%D0%B8%D0%BB%D0%BE%D0%B6%D0%B5%D0%BD%D0%B8%D0%B5-1)» руководства.
* **Как сохранить лог (транскрипт) игры?**  
Используйте команду ЛОГ / клавишу F8. Игровой лог, доступный для копирования откроется в отдельном окне браузера. Вы можете скопировать весь лог или отдельную его часть. Также имеется возможность сразу сохранить весь лог в файл. Для этого выберите в меню браузера пункт «Сохранить как» (тип файла: «Веб-страница полностью»). Лог игры будет доступен в сохраненном HTML-документе.

## Лицензии и компоненты

protoparser.js is copyright (c) 2018 Alexey Galkin &lt;https://johnbrown90210.gitlab.com/protoparser.js&gt;, released under the [MIT license](../LICENSE.txt).  
terminal.js is copyright (c) 2014 Erik Österberg &lt;https://github.com/eosterberg/terminaljs&gt;, 2018 Alexey Galkin &lt;https://johnbrown90210.gitlab.com/protoparser.js&gt;, released under the [MIT license](../LICENSE.txt).

## Обратная связь

Если вы хотите задать вопрос или сообщить об ошибке в движке или руководстве это можно сделать на форуме [ifiction.ru](http://forum.ifiction.ru/viewtopic.php?id=2342), либо создав issue на странице проекта https://gitlab.com/johnbrown90210/protoparser.js/issues.





